package pbcviewer;

import java.io.DataInputStream;
import java.io.IOException;
import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import javax.microedition.io.Connector;
import javax.microedition.io.HttpConnection;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

public class ComicCanvas extends Canvas {
	// reference to self for imbedded classes to use
	protected static Canvas myself = null; 
	
	// constants: names of static menu options
	public static final String LABEL_FIRST = "First Comic";
	public static final String LABEL_PREVIOUS = "Previous Comic";
	public static final String LABEL_RANDOM = "Random Comic";
	public static final String LABEL_NEXT = "Next Comic";
	public static final String LABEL_LAST = "Last Comic";
	public static final String LABEL_NOTE = "Today's Note";
	public static final String LABEL_BACK = "Main Menu";
	public static final String LABEL_HELP = "Help";
	public static final String LABEL_ABOUT = "About";
	public static final String LABEL_DAYOPTION = "View Day";
	public static final String LABEL_RETRY = "Retry";

	protected PBCViewer mainProgram = null;
	protected Vector displayPics = null;
	protected Vector displayPicURLs = null;
	protected int comicRunID = 0; // part of a fix for when comic is loaded while another's loading
	
	// first, previous, next, last comics, for navigation
	protected String first = "-1";
	protected String previous = "-1";
	protected String current = "-1";
	protected String next = "-1";
	protected String last = "-1";
	protected String attempting = "";
	protected String note = ""; // optional note
	protected String comicID = ""; // comic ID, rare but hey...gotta find images somehow
	
	// comic name, url
	protected String comicName = "";
	protected String comicURL = ""; // dir for .php file
	protected String comicDirURL = ""; // dir for comic images
	
	// show scrollbars?
	protected boolean hScrollbar = false;
	protected boolean vScrollbar = false;
	
	// is this screen loading? if so, say so! Default is true.
	//protected boolean loading = true;
	
	// data grabbing thread
	protected DataGrabber dataGrab = null;
	
	// hold the offset values for where img is being displayed
	// also need combined image height, width
	protected Point offset;
	// ok, this one's a misnomer. it's actually just the height of the highest image.
	protected int combinedImageHeight = 0;
	// this one's really a combined width.
	protected int combinedImageWidth = 0;
	
	// yeah, I'll do the timer J.I.C. of big images that need rapid scrolling.
	protected Timer keyStillDownTimer = null;
	protected int keyStillDownCode = 0;
	
	// the menu!
	protected Menu menu = null;
	protected Menu loadingMenu = null; // this menu activates menuDisabled var is on
									   // only options are back and retry comic
	protected int menuActive = 0;
	protected boolean menuDisabled = true;
	
	// custom alert format
	protected GenericAlertDialog alert = null;
	
	// loading pic
	private Image loadingPic = null;
	
	// since there's no need for multiple parallel connections with this system,
	// lets have one http connection + data stream, and have it closed by any subsequent threads that
	// open, hopefully to prevent deadlocks that happen with the sanyo
	protected HttpConnection hConn = null;
    protected DataInputStream inputStream = null;
    
    private boolean thisRunIDOK(int runID) { return runID == comicRunID; }
	
	public Image getRemoteData(String url, int runID) throws IOException, NullPointerException, Exception
	{
        try {
        	// try and get data from url
        	int hCode = 0;
        	
        	//System.out.println("before " + url);
        	
        	// thread-safe code has to be easier than this...
        	// but WHO CARES, IT WORKS!
        	// WOOOOOOOOOOHOOOOOOOOO!!!!!!!!!
        	
        	try {
        		if (!thisRunIDOK(runID)) return null; else
	        	hConn = (HttpConnection)Connector.open(url);
	        	if (!thisRunIDOK(runID)) return null; else
	        	hCode = hConn.getResponseCode();
	        	if (!thisRunIDOK(runID)) return null;
        	} catch (Exception ex) {
        		if (!thisRunIDOK(runID)) return null; else
        		throw ex;
        	}
        	
        	//System.out.println("after " + url);
        	// guess anything but "OK" will be an error, so...
        	if (hCode != 200) {
        		if (!thisRunIDOK(runID)) return null; else
        		throw new IOException("HTTP Error: " + hCode + "\nURL: " + url);
        	}

        	// get info, make sure image is valid
        	int length = 0;
        	String type = "";
        	try {
        		if (!thisRunIDOK(runID)) return null; else
	            length = (int)hConn.getLength();
        		if (!thisRunIDOK(runID)) return null; else
	            type = hConn.getType();
	            if (!type.equals("image/png") && !type.equals("image/gif") && !type.equals("image/jpeg") && !type.startsWith("text/")) {
	            	if (!thisRunIDOK(runID)) return null; else
	            	throw new IOException("Invalid type: " + type);
	            }
        	} catch (Exception ex) {
        		if (!thisRunIDOK(runID)) return null; else
        		throw ex;
        	}

            // there's gotta be something better than array for this
            // arrays are a bit limited, esp w/ a cell phone
            if (length > 0) {
            	byte[] data;
            	try {
            		if (!thisRunIDOK(runID)) return null; else
	            	inputStream = hConn.openDataInputStream();
            		if (!thisRunIDOK(runID)) return null; else
	            	data = new byte[length];
            		if (!thisRunIDOK(runID)) return null; else
	            	inputStream.readFully(data);
            		if (!thisRunIDOK(runID)) return null;
            	} catch (Exception ex) {
            		if (!thisRunIDOK(runID)) return null; else
            		throw ex;
            	}
            	
            	if (type.startsWith("text/"))
            	{
            		if (!thisRunIDOK(runID)) return null; else
            		throw new IOException(new String(data));
            	}
            	else
            	{
            		Image piccy = null;
            		try {
            			if (!thisRunIDOK(runID)) return null; else
            			piccy = Image.createImage(data, 0, length);
            			if (!thisRunIDOK(runID)) return null; else
            			return piccy;
            		} catch (Exception ex) {
            			return null;
            		}
            	}
            } else {
            	if (!thisRunIDOK(runID)) return null; else
            	throw new IOException("Can't display 0 byte files.");
            }
        } finally {
			mainProgram.addDebugRecord("closing connections if kosher");
        	
        	// close connections ONLY if runID is current
        	// else it may close another connection going
        	if (runID == comicRunID)
        		closeConnections(runID);
        	
    		mainProgram.addDebugRecord("exiting getRemoteData");
        }
	}
	
	protected synchronized void closeConnections(int runID)
	{
		mainProgram.addDebugRecord("entering close connections");		
		
		try {
	    	// close connections
	    	if (inputStream != null)
	    	{
	    		mainProgram.addDebugRecord("input stream not null");
	    		
	    		if (runID == comicRunID)
	    			inputStream.close();
	            
	    		mainProgram.addDebugRecord("input closed");
	    	}
		} catch (Exception ex) {
			mainProgram.addDebugRecord("error closing input");
			System.out.println(ex);
		} finally {
			mainProgram.addDebugRecord("before input null");
			if (runID == comicRunID)
				inputStream = null;
			mainProgram.addDebugRecord("after input null");
		}
		
		mainProgram.addDebugRecord("between blocks");
		
		try {
	        if (hConn != null)
	        {
	    		mainProgram.addDebugRecord("hConn null");
	    		if (runID == comicRunID)
	    			hConn.close();
	    		mainProgram.addDebugRecord("hConn closed");
	        }
		} catch (Exception ex) {
			mainProgram.addDebugRecord("hConn error closed");
			System.out.println(ex);
		} finally {
			mainProgram.addDebugRecord("hConn = null");
			if (runID == comicRunID)
				hConn = null;
			mainProgram.addDebugRecord("successful null hConn");
		}
		
		mainProgram.addDebugRecord("leaving closeConnections()");
	}
	
	// simple thread for listening to incoming data and adding it to text window
	private class DataGrabber extends Thread {
		private String url;
		private int vectorID; // holds position in vector that an image goes
		private int runID; // randomly generated value whenever grabComic() invoked
		public DataGrabber(String u, int vID, int rID)
		{ url = u; vectorID = vID; // System.out.println("DataGrabber("+u+","+vID+");");
		  runID = rID; }
		
		public boolean runIDOK()
		{
			if (runID != comicRunID) { System.out.println("runID != comicRunID"); }
			return runID == comicRunID;
		} 

		public void run() {
			// as if. ::shrug::
			if (url == null) return;
			
			Image piccy = null;
			String resultText = "";
			
			// close any currently existing connection
			closeConnections(runID);
			
			// replacement for .swf file
			if (url.toLowerCase().endsWith(".swf"))
			{
				try { piccy = Image.createImage("/swfbad.png"); }
				catch (Exception ex) { failDie("Could not open swfbad.png."); return; }
			}
			else if (url.toLowerCase().endsWith(".php") && !url.endsWith("mobilegetcomic.php"))
			{
				try { piccy = Image.createImage("/phpbad.png"); }
				catch (Exception ex) { failDie("Could not open phpbad.png."); return; }
			}
			else
			{
				mainProgram.addDebugRecord("About to get data/img");
				try { piccy = getRemoteData(url, runID); }
				catch (Exception ex) { resultText = ex.getMessage(); piccy = null; }
				mainProgram.addDebugRecord(((resultText.equals("")) ? "resultText blank" : "resultText not blank") + ((piccy == null) ? " piccy null" : " piccy not null"));
			}
			
			// the reason for runID...if it doesn't equal the canvas's comicRunID
			// then the user has changed to another day's comic. die here.
			if (!runIDOK()) return;
			
			// not likely, but...
			if (resultText == null) resultText = "";
			
			// damnit, damnit, LG LX570 had to be a rebel
			// gotta squeeze the useful packet out of the required html now
			if (!resultText.equals(""))
			{
				mainProgram.addDebugRecord("resultText stripping html");
				// look for text between both <body>'s
				int startPos = Utilities.strpos(resultText, "<body>");
				int endPos = -1;
				if (startPos != -1)
					endPos = Utilities.strpos(resultText, "</body>", startPos + 2);
				
				if (!(startPos == -1 || endPos == -1))
				{
					resultText = resultText.substring(startPos + 6, endPos);
				}
				
				//System.out.println(resultText);
			}
			
			// still not likely, but...
			if (resultText == null) resultText = "";
			
			// gotta be an image, so add it to the vector and run another if necessary
			if (resultText.equals(""))
			{	
				// common issue: know it happens with Samsung + .jpegs
				// piccy is null and no error message thrown.
				// therefore it'll have to display a "can't display" picture in its
				// place. png's usually safe. heh.
				if (piccy == null)
				{
					if (url.toLowerCase().endsWith(".jpg") || url.toLowerCase().endsWith(".jpeg"))
					{
						try { piccy = Image.createImage("/jpgbad.png"); }
						catch (Exception ex) { failDie("Could not open jpgbad.png."); return; }						
					}
					else if (url.toLowerCase().endsWith(".gif"))
					{
						try { piccy = Image.createImage("/gifbad.png"); }
						catch (Exception ex) { failDie("Could not open gifbad.png."); return; }	
					}
					else
					{
						try { piccy = Image.createImage("/imgbad.png"); }
						catch (Exception ex) { failDie("Could not open pngbad.gif."); return; }						
					} // generic?
				}
				
				try {
					if (runIDOK())
						displayPics.addElement(piccy);
				} catch (Exception ex)
				{
					failDie("NPE at adding bad image to vector: " + ex);
					return;
				} 
				
				// display first showing image
				if (runIDOK())
					checkScrollbars();
				if (runIDOK())
					repaint();
				if (displayPics.size() >= displayPicURLs.size())
				{
					if (runIDOK())
						doFinalActions(); // do final actions!
					return;
				}
				
				// otherwise, time pull in the next image :)
				if (runIDOK())
					dataGrab = new DataGrabber(comicDirURL + comicID + "/" + (String) displayPicURLs.elementAt(vectorID + 1), vectorID + 1, runID);
				if (runIDOK())
					dataGrab.start();
			} // otherwise, create an ImageCanvas to display the pic!
			else if (Utilities.strpos(resultText, "SUCCESS") != -1)
			{
				//System.out.println(resultText);
				// parse the string.
				int j = 0;
				
				// get comic ID
				int startPos = Utilities.strpos(resultText, "<p>", j);
				int endPos = -1;
				if (startPos != -1)
					endPos = Utilities.strpos(resultText, "</p>", startPos + 2);
				if (!(startPos == -1 || endPos == -1))
				{
					comicID = resultText.substring(startPos + 3, endPos);
					j = endPos + 2;
				}
				if (comicID.equals("-1")) comicID = "";
				if (runIDOK())
					comicID = new String(comicID + "comic_images/comics");
				//System.out.println("comicID: " + comicID + " j: " + j + " length: " + comicID.length());
				
				// get first comic date
				startPos = Utilities.strpos(resultText, "<p>", j);
				endPos = -1;
				if (startPos != -1)
					endPos = Utilities.strpos(resultText, "</p>", startPos + 2);
				if (!(startPos == -1 || endPos == -1))
				{
					if (runIDOK())
						first = resultText.substring(startPos + 3, endPos);
					j = endPos + 2;
				}
				//System.out.println("first: " + first + " j: " + j);
				
				// get previous comic date
				startPos = Utilities.strpos(resultText, "<p>", j);
				endPos = -1;
				if (startPos != -1)
					endPos = Utilities.strpos(resultText, "</p>", startPos + 2);
				if (!(startPos == -1 || endPos == -1))
				{
					if (runIDOK())
						previous = resultText.substring(startPos + 3, endPos);
					j = endPos + 2;
				}
				//System.out.println("previous: " + previous + " j: " + j);
				
				// get current comic date
				startPos = Utilities.strpos(resultText, "<p>", j);
				endPos = -1;
				if (startPos != -1)
					endPos = Utilities.strpos(resultText, "</p>", startPos + 2);
				if (!(startPos == -1 || endPos == -1))
				{
					if (runIDOK())
						current = resultText.substring(startPos + 3, endPos);
					j = endPos + 2;
				}
				//System.out.println("current: " + current + " j: " + j);
				
				// FIX TO RANDOM/RETRY BUG: make "attempting" = current
				if (runIDOK())
					attempting = new String(current);
				
				// get next comic date
				startPos = Utilities.strpos(resultText, "<p>", j);
				endPos = -1;
				if (startPos != -1)
					endPos = Utilities.strpos(resultText, "</p>", startPos + 2);
				if (!(startPos == -1 || endPos == -1))
				{
					if (runIDOK())
						next = resultText.substring(startPos + 3, endPos);
					j = endPos + 2;
				}
				//System.out.println("next: " + next);
				
				// get last comic date
				startPos = Utilities.strpos(resultText, "<p>", j);
				endPos = -1;
				if (startPos != -1)
					endPos = Utilities.strpos(resultText, "</p>", startPos + 2);
				if (!(startPos == -1 || endPos == -1))
				{
					if (runIDOK())
						last = resultText.substring(startPos + 3, endPos);
					j = endPos + 2;
				}
				//System.out.println("last: " + last);
				
				// get num comic image files
				String numImagesStr = "";
				startPos = Utilities.strpos(resultText, "<p>", j);
				endPos = -1;
				if (startPos != -1)
					endPos = Utilities.strpos(resultText, "</p>", startPos + 2);
				if (!(startPos == -1 || endPos == -1))
				{
					numImagesStr = resultText.substring(startPos + 3, endPos);
					j = endPos + 2;
				}
				
				// now turn it into a number. if this fails then die
				int numImages = 0;
				try { numImages = Integer.parseInt(numImagesStr);
				} catch (Exception ex) { failDie("Invalid comic data received from server. Cannot load comic. " + resultText + " " + first + " " + previous + " " + current + " " + next + " " + last + " " + numImagesStr); return; }
				//System.out.println("numImages: " + numImages);
				
				// die if there's 0 comics to display
				if (numImages <= 0)
				{
					failDie("Packet received from the server successfully, but there are no comics to display. Please notify the artist of this error.");
					return;
				}
				
				// grab and place into vector the image url's as needed
				for (int i = 0; i < numImages; i++)
				{
					String imageURL = "";
					startPos = Utilities.strpos(resultText, "<p>", j);
					endPos = -1;
					if (startPos != -1)
						endPos = Utilities.strpos(resultText, "</p>", startPos + 2);
					if (!(startPos == -1 || endPos == -1))
					{
						imageURL = resultText.substring(startPos + 3, endPos);
						j = endPos + 2;
					}
					
					// die if imageURL is blank
					if (imageURL.equals(""))
					{
						failDie("Missing an image URL. Cannot load comic.");
						return;
					}
					
					// add url to vector
					if (runIDOK())
						displayPicURLs.addElement(new String(imageURL));
					//System.out.println("Image URL #" + i + ": " + imageURL);
				}
				//System.out.println("displayPicURLs OK");
				
				// finally bring in note. this is optional, may be blank.
				startPos = Utilities.strpos(resultText, "<p>", j);
				endPos = -1;
				if (startPos != -1)
					endPos = Utilities.strpos(resultText, "</p>", startPos + 2);
				if (!(startPos == -1 || endPos == -1))
				{
					if (runIDOK())
						note = resultText.substring(startPos + 3, endPos);
					j = endPos + 2;
				}
				
				if (runIDOK() && note.equals("-1")) note = "";
				//System.out.println("note: " + note);
				
				// create menu
				if (runIDOK())
					createMenu();
				
				// pull in first image!
				if (runIDOK())
					dataGrab = new DataGrabber(comicDirURL + comicID + "/" + (String) displayPicURLs.elementAt(0), 0, runID);
				if (runIDOK())
					dataGrab.start();
			}
			else // failure/error
			{
				alert = new GenericAlertDialog(myself, mainProgram, "Could not load " + comicName + ": " + resultText, getWidth() / 8 * 7, true, false);
				repaint();
				return;
			}
		}
	}
	
	// something failed, so die.
	private void failDie(String error)
	{
		Alert a = new Alert("Error");
		a.setString(error);
		a.setTimeout(Alert.FOREVER);
		mainProgram.getDisplay().setCurrent(a, mainProgram.getMainMenu()); // back to main menu
		//mainProgram.getMainMenu().setKeyBlock(false);
		mainProgram.getMainMenu().setComicCanvas(null); // to be picked up by GC
	}
	
	// the nullify option is for when a connection goes fubar, happens alot with sanyo 8400
	public void wrapUp(boolean nullify)
	{
		mainProgram.getDisplay().setCurrent(mainProgram.getMainMenu()); // back to main menu
		//mainProgram.getMainMenu().setKeyBlock(false);
		if (nullify)
			mainProgram.getMainMenu().setComicCanvas(null); // to be picked up by GC		
	}
	
	private class ArrowTimerTask extends TimerTask {
		public ArrowTimerTask()
		{
			super();
		}

		public void run() {
			// action key code. it can be multiple. THIS allows up-left etc.
			if (keyStillDownCode == Canvas.UP)
				doUpAction();
			else if (keyStillDownCode == Canvas.DOWN)
				doDownAction();
			else if (keyStillDownCode == Canvas.LEFT)
				doLeftAction();
			else if (keyStillDownCode == Canvas.RIGHT)
				doRightAction();
		}
	}
	
	// takes in calling program, comic name, comic URL
	public ComicCanvas(PBCViewer h, String cN, String cURL)
	{
		super();
		
		myself = this;
		
		try {
			loadingPic = Image.createImage("/loading.png");
		} catch (Exception ex) {
			loadingPic = null;
		}
		
		mainProgram = h;
		comicName = cN;
		comicURL = cURL;
		
		// this menu must always exist, it'll display while the comic is loading
		if (loadingMenu != null) loadingMenu = null; // will this happen?
		loadingMenu = new Menu(this, mainProgram);
		
		// only two static options: Retry and Main Menu
		loadingMenu.addOption(LABEL_RETRY, false, false, false);
		loadingMenu.addOption(LABEL_BACK, false, false, false);
		
		// MAKE THE GODDAMN VECTORS! SHEESH!
		displayPics = new Vector();
		displayPicURLs = new Vector();
		
		// figure out the comic dir URL...in spite of the nerfed String functions :(
		// does url have ~ char? if so, trim it at the fourth /
		// otherwise, trim at third /
		int slashCount = 3;
		if (comicURL.indexOf('~') != -1) slashCount = 4;
		
		int endPos = 0;
		for (int i = 0; i < slashCount; i++)
		{
			endPos = comicURL.indexOf('/', endPos) + 1; 
		}
		
		// rare happening, url is fake...it's just http://
		if (endPos <= 0)
		{
			comicDirURL = "http://";
		}
		else
		{
			String cuTemp = comicURL.substring(0, endPos - 1);
			if (cuTemp.endsWith("/")) comicDirURL = cuTemp;
			else comicDirURL = cuTemp + "/";
		}
		
		// add back command
		mainProgram.addBackCommand(this);
		
		grabComic("", false);
	}
	
	protected void grabComic(String day, boolean rand)
	{
		if (day == null) day = "";
		
		// create a comic run ID. this prevents old threads from lagged load
		// attempts from interfering with anything. if a thread has the wrong comicRunID,
		// that thread will just stop running once it gets somewhere.
		// do it first in the off-chance of some threading issue.
		comicRunID = (new Random()).nextInt();
		
		// set as "attempting" comic, so user can try to load comic again if their
		// connection dies, and not get frozen forever.
		attempting = new String(day);
		
		String dayString = (day.equals("") ? "" : ("?day=" + day));
		String versionString = (day.equals("") ? ("?version=" + PBCViewer.SERVER_VERSION_SUPPORT) : ("&version=" + PBCViewer.SERVER_VERSION_SUPPORT));
		String randomString = (rand ? "&random=1" : "");
		
		// reset what needs to be reset
		displayPics.removeAllElements();
		displayPicURLs.removeAllElements();
		combinedImageHeight = 0;
		combinedImageWidth = 0;
		offset = null;
		offset = new Point(0, 0, 0, 0, 0, 0);
		current = "-1";
		
		// repaint so loading... msg appears again
		repaint();
		
		// still need to decide if this comic's viewable.
		// first, make sure the URL is valid.
		menuDisabled = true; // disable menu until menu contents are known
		
		dataGrab = new DataGrabber(comicURL + "mobilegetcomic.php" + dayString + versionString + randomString, -1, comicRunID);
		dataGrab.start();
	}

	// final actions to do after images are loaded successfully.
	public void doFinalActions()
	{
		// determine now if we'll need scrollbars. this'll help determine the offsets
		checkScrollbars();
		
		// enable full menu
		menuDisabled = false;
		
		//loading = false;
		dataGrab = null;
		repaint();
	}
	
	private synchronized void checkScrollbars()
	{
		// determine the total x-y needed
		int cih = 0; // combined image height
		int ciw = 0; // combined image width
		for (int i = 0; i < displayPics.size(); i++)
		{
			Image img = (Image) displayPics.elementAt(i);
			try {
				if (img.getHeight() > cih) // because all pics are lined up horizontally
					cih = img.getHeight(); // just get the highest height
				ciw += img.getWidth();
			} catch (Exception ex) {
				failDie("NPE at checkScrollbars(): " + ex);
				return;
			}
		}
		
		combinedImageHeight = cih;
		combinedImageWidth = ciw;
		
		// reset these
		hScrollbar = false;
		vScrollbar = false;
		
		// do we need scrollbars?
		if (combinedImageWidth > getWidth())
			hScrollbar = true;
		if (combinedImageHeight > getHeight())
			vScrollbar = true;
		
		// now see if one scrollbar makes it so the other needs a scrollbar.
		if (hScrollbar && !vScrollbar)
		{
			if (combinedImageHeight > getHeight() - Scrollbar.WIDTH)
				vScrollbar = true; 
		}
		else if (vScrollbar && !hScrollbar)
		{
			if (combinedImageWidth > getWidth() - Scrollbar.WIDTH)
				hScrollbar = true;
		}
		
		
		int vmod = hScrollbar ? Scrollbar.WIDTH : 0;
		int hmod = vScrollbar ? Scrollbar.WIDTH : 0;
		
		// base the offsets off previous, since this'll be called whenever an image loads
		offset = new Point(offset.getX(),offset.getY(), getWidth() - combinedImageWidth - hmod, getHeight() - combinedImageHeight - vmod, 0, 0);
	}
	
	// creates the menu, automatically deleting the old one if applicable.
	// call this after packet received.
	protected synchronized void createMenu()
	{
		if (menu != null) menu = null; // garbage!
		menu = new Menu(this, mainProgram);
		
		menu.addOption(LABEL_FIRST, first.equals("-1"), false, false);
		menu.addOption(LABEL_PREVIOUS, previous.equals("-1"), false, false);
		menu.addOption(LABEL_NEXT, next.equals("-1"), false, false);
		menu.addOption(LABEL_LAST, last.equals("-1"), false, false);
		menu.addOption(Menu.MENU_BREAK, true, false, false);
		menu.addOption(LABEL_NOTE, note.equals(""), false, false);
		menu.addOption(LABEL_RANDOM, false, false, false);
		menu.addOption(LABEL_DAYOPTION, false, true, mainProgram.dateShown());
		menu.addOption(Menu.MENU_BREAK, true, false, false);
		menu.addOption(LABEL_HELP, false, false, false);
		menu.addOption(LABEL_ABOUT, false, false, false);
		
		// back to main menu option
		menu.addOption(LABEL_BACK, false, false, false);
		
		// finally, enable menu...it's only disabled because it may not exist yet.
		//menuDisabled = false;
	}

	protected void paint(Graphics g) {
		try {
		if (displayPics == null) return;
		
		g.setColor(0,0,0);
		g.fillRect(0, 0, getWidth(), getHeight());
		
		// paint loading message if loading.
		if (displayPics.size() == 0)
		{
			if (loadingPic != null)
				g.drawImage(loadingPic, getWidth() / 2, (getHeight() / 2) - (loadingPic.getHeight() / 2) , Graphics.HCENTER | Graphics.TOP);
			
			// draw menu w/ retry, etc. options
			if (menuActive == 2 && loadingMenu != null)
			{
				loadingMenu.paint(g, 0);
			}
			
			// draw alert if applicable
			if (alert != null)
				alert.drawDialog(g);
			
			return;
		}
		
		// offset for what's already been drawn
		int widthOffset = 0;

		// imma start being lazy and draw all images, but then fix later
		for (int i = 0; i < displayPics.size(); i++)
		{
			try {
				Image img = (Image)displayPics.elementAt(i);
				
				// don't draw offscreen images
				// first possibility: image is completely offscreen to the left
				// second possibility: image is completely offscreen to the right
				// not using "continue" because adding widthOffset would get complicated
				if (img.getWidth() + widthOffset <= offset.getX() * -1) { }
				else if (widthOffset > getWidth() + (offset.getX() * -1)) { }
				else
					g.drawImage(img, offset.getX() + widthOffset, offset.getY(), Graphics.TOP | Graphics.LEFT);
				
				widthOffset += img.getWidth();
			} catch (Exception ex) {
				failDie("NPE at attempt to draw image: " + ex);
				return;
			}
		}
		
		// draw scrollbars if applicable
		if (hScrollbar && offset != null)
		{
			Scrollbar.drawScrollbar(g, getWidth(), getHeight(), combinedImageWidth,
					getWidth() - (vScrollbar ? Scrollbar.WIDTH : 0), offset.getX() * -1, 0, 0, 0, 0, false, vScrollbar);
		}
		if (vScrollbar && offset != null)
		{
			Scrollbar.drawScrollbar(g, getWidth(), getHeight(), combinedImageHeight,
					getHeight() - (hScrollbar ? Scrollbar.WIDTH : 0), offset.getY() * -1, 0, 0, 0, 0, true, hScrollbar);
		}
		
		// draw periodical loading msg if applicable
		if (displayPics.size() < displayPicURLs.size())
		{
			String loadMsg = "Loading images: " + displayPics.size() + " of " + displayPicURLs.size();
			int loadMsgWidth = loadMsg.length() * 5;
			g.setColor(0,0,0);
			g.drawRect((getWidth() / 2) - (loadMsgWidth / 2) - 3, getHeight() - 13, loadMsgWidth + 3, 13);
			g.setColor(255,255,255);
			g.fillRect((getWidth() / 2) - (loadMsgWidth / 2) - 2, getHeight() - 12, loadMsgWidth + 2, 12);
			g.setColor(0, 0,0);
			Utilities.drawString(g, loadMsg, (getWidth() / 2), getHeight() - 10, Graphics.TOP | Graphics.HCENTER, mainProgram, Font.SIZE_SMALL);
		}
		
		// draw date if applicable
		if (mainProgram.dateShown())
		{
			String todayString = Utilities.humanFormatDate(current);
			
			// only draw if it's valid
			if (!todayString.equals(""))
			{	
				// background, draw rect at -1 so rect appears to come out of top
				g.setColor(0, 0, 0);
				g.drawRect((getWidth() / 2) - 23, -1, 43, 10);
				g.setColor(255,255,255);
				g.fillRect((getWidth() / 2) - 22, 0, 42, 9);
				g.setColor(0,0,0);
				Utilities.drawString(g, todayString, getWidth() / 2 + 1, 1, Graphics.TOP | Graphics.HCENTER, mainProgram, Font.SIZE_SMALL);
			}
		}
		
		// draw menu if applicable. menu should ALWAYS be second to last in the paint list.
		if (menuActive == 1 && menu != null)
		{
			menu.paint(g, 0);
		}
		else if (menuActive == 2 && loadingMenu != null)
		{
			loadingMenu.paint(g, 0);
		}
		
		// draw alert if applicable
		if (alert != null)
			alert.drawDialog(g);
		} catch (Exception ex) {
			failDie(ex.toString() + ex.getMessage());
		}
	}
	
	// Up, down, left, right actions
	protected void doUpAction()
	{
		if (offset == null) return;
		
		offset.setY(offset.getY() + (getHeight() / 3));
		repaint();
	}
	protected void doDownAction()
	{
		if (offset == null) return;
		
		offset.setY(offset.getY() - (getHeight() / 3));
		repaint();
	}
	protected void doLeftAction()
	{
		if (offset == null) return;
		
		offset.setX(offset.getX() + (getWidth() / 3));
		repaint();
	}
	protected void doRightAction()
	{
		if (offset == null) return;
		
		offset.setX(offset.getX() - (getWidth() / 3));
		repaint();
	}
	
	protected void alertActiveKeyPressed(int keyCode, int oldKeyCode)
	{
		// J.I.C.
		if (alert == null) { repaint(); return; }
		
		if (keyCode == Canvas.FIRE)
		{
			alert = null;
			repaint();
		}
		else if (keyCode == Canvas.UP)
		{
			if (alert.upPressed()) repaint();
		}
		else if (keyCode == Canvas.DOWN)
		{
			if (alert.downPressed()) repaint();
		}
	}
	
	protected void loadingMenuKeyPressed(int keyCode, int oldKeyCode)
	{
		// should never happen, but need to be safe...
		if (loadingMenu == null) return;
		
		if (keyCode == Canvas.UP)
		{
			loadingMenu.upOption();
			repaint();
		}
		else if (keyCode == Canvas.DOWN)
		{
			loadingMenu.downOption();
			repaint();
		}
		else if (keyCode == Canvas.FIRE)
		{
			// stuff for each option
			if (loadingMenu.getText().equals(LABEL_RETRY))
			{
				menuActive = 0;
				repaint();
				grabComic(attempting, false);
				return;
			}
			else if (loadingMenu.getText().equals(LABEL_BACK))
			{
				wrapUp(true);
				menuActive = 0;
				return;
			}
		}
		else if (oldKeyCode == Canvas.KEY_POUND || oldKeyCode == Canvas.KEY_STAR)
		{				
			menuActive = 0;
			repaint();
		}
	}
	
	protected void menuActiveKeyPressed(int keyCode, int oldKeyCode)
	{
		// fail if no menu
		if (menu == null) return;
		
		if (keyCode == Canvas.UP)
		{
			menu.upOption();
			repaint();
		}
		else if (keyCode == Canvas.DOWN)
		{
			menu.downOption();
			repaint();
		}
		else if (keyCode == Canvas.FIRE)
		{
			// stuff for each option
			if (menu.getText().equals(LABEL_FIRST))
			{
				menuActive = 0;
				repaint();
				grabComic(first, false);
				menu = null;
				return;
			}
			else if (menu.getText().equals(LABEL_PREVIOUS))
			{
				menuActive = 0;
				repaint();
				grabComic(previous, false);
				menu = null;
				return;
			}
			else if (menu.getText().equals(LABEL_RANDOM))
			{
				menuActive = 0;
				repaint();
				grabComic("-1", true);
				menu = null;
				return;
			}
			else if (menu.getText().equals(LABEL_NEXT))
			{
				menuActive = 0;
				repaint();
				grabComic(next, false);
				menu = null;
				return;
			}
			else if (menu.getText().equals(LABEL_LAST))
			{
				menuActive = 0;
				repaint();
				grabComic(last, false);
				menu = null;
				return;
			}
			else if (menu.getText().equals(LABEL_HELP))
			{
				menuActive = 0;
				alert = new GenericAlertDialog(this, mainProgram, "Press direction keys to scroll through comics.\n"
						+ "Press * to open menu. View note views today's note if applicable.\n"
						+ "1 and 3 take you to the previous and next comics respectively.\n"
						+ "Random comic takes you to a random comic if the artist allows random browsing.\n"
						+ "View day toggles day display. Main menu takes you back.", (getWidth() / 8) * 7, false, false);
				repaint();
			}
			else if (menu.getText().equals(LABEL_ABOUT))
			{
				menuActive = 0;
				alert = mainProgram.createAboutDialog(this);
				repaint();
			}
			else if (menu.getText().equals(LABEL_NOTE))
			{
				menuActive = 0;
				alert = new GenericAlertDialog(this, mainProgram, note, (getWidth() / 8) * 7, false, false);
				repaint();
			}
			else if (menu.getText().equals(LABEL_BACK))
			{
				wrapUp(false);
				menuActive = 0;
				return;
			}
			else if (menu.getText().equals(LABEL_DAYOPTION))
			{
				menu.checkOption();
				mainProgram.setShowDate(menu.currentOptionChecked());
				repaint();
				return;
			}
		}
		else if (oldKeyCode == Canvas.KEY_POUND || oldKeyCode == Canvas.KEY_STAR)
		{				
			menuActive = 0;
			repaint();
		}
	}
	
	protected void normalKeyPressed(int keyCode, int oldKeyCode)
	{
		// change and repaint
		if (keyCode == Canvas.UP)
			doUpAction();
		else if (keyCode == Canvas.DOWN)
			doDownAction();
		else if (keyCode == Canvas.LEFT)
			doLeftAction();
		else if (keyCode == Canvas.RIGHT)
			doRightAction();
		
		if (keyCode == Canvas.UP || keyCode == Canvas.DOWN || keyCode == Canvas.LEFT || keyCode == Canvas.RIGHT)
		{
			killTimer();
			keyStillDownCode = keyCode;
			keyStillDownTimer = new Timer();
			keyStillDownTimer.schedule(new ArrowTimerTask(), 1000, 500);
		}
		
		// previous comic when 1 is pressed
		if (oldKeyCode == Canvas.KEY_NUM1 && !previous.equals("-1"))
		{
			grabComic(previous, false);
			menu = null;
		}
		else if (oldKeyCode == Canvas.KEY_NUM3 && !next.equals("-1"))
		{
			grabComic(next, false);
			menu = null;
		} // do something for keyCode back, so i can get out of this
		else if (keyCode == Canvas.FIRE || oldKeyCode == Canvas.KEY_POUND || oldKeyCode == Canvas.KEY_STAR)
		{
			// gotta kill the timer
			killTimer();
			
			// load proper menu for situation
			if (menuDisabled || menu == null)
			{
				if (loadingMenu == null)
					return; // blah blah
				else
					menuActive = 2;
			}
			else
				menuActive = 1;
			
			repaint();
		}
	}
	
	protected void keyPressed(int keyCode)
	{	
		//if (loading) return; // don't let this happen while loading
		
		int oldKeyCode = keyCode;
		
		// must always change this into a game action code
		keyCode = getGameAction(keyCode);
		
		if (alert != null) alertActiveKeyPressed(keyCode, oldKeyCode);
		else if (menuActive == 1) menuActiveKeyPressed(keyCode, oldKeyCode);
		else if (menuActive == 2) loadingMenuKeyPressed(keyCode, oldKeyCode);
		else normalKeyPressed(keyCode, oldKeyCode); 
	}
	
	protected void keyReleased(int keyCode)
	{
		// must always change this into a game action code
		keyCode = getGameAction(keyCode);
		
		if (keyStillDownCode == keyCode)
			killTimer();
	}
	
	protected void killTimer()
	{
		if (keyStillDownTimer != null)
		{
			keyStillDownTimer.cancel();
			keyStillDownTimer = null;
		}
	}
	
	public String getComicName() { return comicName; }
	public String getComicURL() { return comicURL; }
	
	
	public boolean menuIsOpen()
	{
		if (menuActive == 0)
			return false;
		
		return true;
	}
	
	public void closeMenu()
	{
		menuActive = 0;
		repaint();
	}
}