package pbcviewer;

import java.util.Vector;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

public class GenericAlertDialog extends AlertDialog {
	private static final int LINE_HEIGHT = 10;
	private Vector lines = null;
	int width;
	int height;
	private boolean icon = false;
	private boolean confirm = false;
	private Image piccy = null;
	private boolean yesSelected = false;
	
	private int linesScrolledDown = 0;
	private int screenMaxLines = 0;
	private int numLines = 0;
	
	public GenericAlertDialog(Canvas c, PBCViewer mp, String s, int w, boolean alertIcon,
								boolean confirmDialog)
	{
		super("", c, mp);
		lines = new Vector();
		width = w;
		
		Utilities.strFillVector(s, width - 2, lines);
		
		numLines = lines.size();
		
		// get screen max # lines lines
		screenMaxLines = (c.getHeight() - 8) / LINE_HEIGHT;
		
		// additional features
		icon = alertIcon;
		confirm = confirmDialog;
		
		if (icon)
		{
			try { piccy = Image.createImage("/alert.png"); numLines += 3;}
			catch (Exception ex) { piccy = null; icon = false; }
		}
		
		height = numLines * LINE_HEIGHT;
		
		height += 6;
		
		if (height > c.getHeight()) height = c.getHeight() - 6;
		
		// go through each lines and set the width to the length of the longest line
		int longLine = 0;
		for (int i = 0; i < lines.size(); i++)
		{
			String sssss = (String) lines.elementAt(i);
			
			if (sssss == null) continue;
			
			if (sssss.length() > longLine)
				longLine = sssss.length();
		}
		
		width = longLine * 5 + 2;
		
		if (confirm)
		{
			// #1: increase height by 20, #2: don't let width be < 80
			if (width < 80) width = 80;
			height += 20;
		}
	}
	
	public void drawDialog(Graphics g)
	{
		if (canvas == null || lines == null || mainProgram == null) return;
		
		g.setColor(240,240,240);
		g.fillRect((canvas.getWidth() / 2) - (width / 2) - 1, (canvas.getHeight() / 2) - (height / 2), width, height);
		g.setColor(0,0,0);
		g.drawRect((canvas.getWidth() / 2) - (width / 2) - 2, (canvas.getHeight() / 2) - (height / 2) - 1, width, height + 1);
		
		// draw icon if applicable
		if (icon && piccy != null)
			g.drawImage(piccy, canvas.getWidth() / 2, (canvas.getHeight() / 2) - (height / 2) + 3, Graphics.TOP | Graphics.HCENTER);

		for (int i = linesScrolledDown; i < lines.size(); i++)
		{
			//System.out.println("i=" + i + " screenMaxLines + linesScrolledDown = " + (screenMaxLines + linesScrolledDown));
			
			if (i == screenMaxLines + linesScrolledDown)
				break; // don't draw more lines than screen can fit
			String text = (String) lines.elementAt(i);
			Utilities.drawString(g, text, canvas.getWidth() / 2, (canvas.getHeight() / 2) - (height / 2) + (10 * ((i - linesScrolledDown) + (icon ? 3 : 0))) + 3, Graphics.TOP | Graphics.HCENTER, mainProgram, Font.SIZE_SMALL);
		}
		
		if (confirm)
		{
			// draw hilight for yes/no
			g.setColor(192, 192, 255);
			if (yesSelected) 
				g.fillRect((canvas.getWidth() / 2) - 41, (canvas.getHeight() / 2) + (height / 2) - 14, 30, 13);
			else
				g.fillRect((canvas.getWidth() / 2) + 8, (canvas.getHeight() / 2) + (height / 2) - 14, 30, 13);
			
			// draw yes/no
			//g.drawString("Yes", (canvas.getWidth() / 2) - 25, (canvas.getHeight() / 2) + 15, Graphics.TOP | Graphics.HCENTER);
			//g.drawString("No", (canvas.getWidth() / 2) + 25, (canvas.getHeight() / 2) + 15, Graphics.TOP | Graphics.HCENTER);
			Utilities.drawString(g, "Yes", (canvas.getWidth() / 2) - 25, (canvas.getHeight() / 2) + (height / 2) - 12, Graphics.TOP | Graphics.HCENTER, mainProgram, Font.SIZE_SMALL);
			Utilities.drawString(g, "No", (canvas.getWidth() / 2) + 25, (canvas.getHeight() / 2) + (height / 2) - 12, Graphics.TOP | Graphics.HCENTER, mainProgram, Font.SIZE_SMALL);
		}
		
		// draw arrows if applicable
		if (linesScrolledDown > 0)
		{
			g.setColor(0, 0, 128);
			g.drawLine((canvas.getWidth() / 2) - 2, (canvas.getHeight() / 2) - (height / 2), (canvas.getWidth() / 2) - 2, (canvas.getHeight() / 2) - (height / 2));
			g.drawLine((canvas.getWidth() / 2) - 3, (canvas.getHeight() / 2) - (height / 2) + 1, (canvas.getWidth() / 2) - 1, (canvas.getHeight() / 2) - (height / 2) + 1);
			g.drawLine((canvas.getWidth() / 2) - 4, (canvas.getHeight() / 2) - (height / 2) + 2, (canvas.getWidth() / 2), (canvas.getHeight() / 2) - (height / 2) + 2);
		}
		if (numLines - screenMaxLines > linesScrolledDown)
		{
			g.setColor(0, 0, 128);
			g.drawLine((canvas.getWidth() / 2) - 2, (canvas.getHeight() / 2) + (height / 2), (canvas.getWidth() / 2) - 2, (canvas.getHeight() / 2) + (height / 2));
			g.drawLine((canvas.getWidth() / 2) - 3, (canvas.getHeight() / 2) + (height / 2) - 1, (canvas.getWidth() / 2) - 1, (canvas.getHeight() / 2) + (height / 2) - 1);
			g.drawLine((canvas.getWidth() / 2) - 4, (canvas.getHeight() / 2) + (height / 2) - 2, (canvas.getWidth() / 2), (canvas.getHeight() / 2) + (height / 2) - 2);
		}
	}
	
	public void setYes(boolean y)
	{
		yesSelected = y;
	}
	
	public boolean getYes()
	{
		return yesSelected;
	}
	
	public String getText() { return text; }
	
	// returns true if needs to be repainted
	public boolean upPressed()
	{
		if (linesScrolledDown > 0)
		{
			linesScrolledDown--; // scroll up one
			return true; // repaint (if calling form uses this to do so)
		}
		
		return false;
	}

	// returns true if needs to be repainted
	public boolean downPressed()
	{
		if (linesScrolledDown < numLines - screenMaxLines)
		{
			linesScrolledDown++; // scroll down one
			return true; // repaint (if calling form uses this to do so)
		}
		
		return false;
	}
}