package pbcviewer;

import java.util.Vector;

import javax.microedition.lcdui.*;

// trying to standardize all the menus in this program...all two of them!
public class Menu extends Object {
	private class MenuElement {
		private boolean faded;
		private String str;
		private boolean checkable;
		private boolean checked = true;
		
		public MenuElement(String s, boolean f, boolean ck, boolean ckd)
		{
			faded = f; str = s; checkable = ck; checked = ckd;
		}
		
		public void setFaded(boolean f) { faded = f; }
		public void setLabel(String s) { str = s; }
		public void setCheckable(boolean c) { checkable = c; }
		public void setChecked(boolean c) { checked = c; }
		public boolean isFaded() { return faded; }
		public String getLabel() { return str; }
		public boolean isCheckable() { return checkable; }
		public boolean isChecked() { return checked; }
	}
	
	private int menuPos = -1;
	private Vector menuOptions = null;
	private Canvas callingForm = null;
	private PBCViewer mainProgram = null;
	
	// menu constants
	public static final int MENU_OPTION_HEIGHT = 11;
	public static final int MENU_OPTION_WIDTH = 120;
	
	// the break constant
	public static final String MENU_BREAK = "---";
	
	// the check image :)
	private Image check;
	private Image nocheck;

	public Menu(Canvas c, PBCViewer mp) {
		menuOptions = new Vector();
		callingForm = c;
		mainProgram = mp;
		
		try { check = Image.createImage("/check.png"); nocheck = Image.createImage("/nocheck.png"); }
		catch (Exception ex) { check = null; nocheck = null; }
	}
	
	public void addOption(String s, boolean faded, boolean ck, boolean ckd)
	{
		if (menuOptions == null) return;
		
		menuOptions.addElement(new MenuElement(s, faded, ck, ckd));
		resetMenu();
	}
	
	public int getSelectedIndex() { return menuPos; }
	public String getText()
	{
		if (menuPos == -1)
			return "";
		if (menuOptions == null) return "";
		
		return ((MenuElement)menuOptions.elementAt(menuPos)).getLabel();
	}
	
	// user presses up
	public void upOption()
	{
		if (menuOptions == null) return;
		
		// if menuPos is -1 then everything's disabled, auto fail
		if (menuPos == -1)
			return;
		else
		{
			// find next valid option if possible
			for (int i = menuPos - 1; i >= 0; i--)
			{
				MenuElement e = (MenuElement) menuOptions.elementAt(i);
				if (!e.isFaded())
				{
					menuPos = i;
					callingForm.repaint();
					return;
				}
			}
			
			// do one more up run, for menu wrapping
			for (int i = menuOptions.size() - 1; i >= 0; i--)
			{
				MenuElement e = (MenuElement) menuOptions.elementAt(i);
				if (!e.isFaded())
				{
					menuPos = i;
					callingForm.repaint();
					return;
				}
			}
		}
	}
	
	// user presses down
	public void downOption()
	{
		if (menuOptions == null) return;
		
		// if menuPos is -1 then everything's disabled, auto fail
		if (menuPos == -1)
			return;
		else
		{
			// find next valid value if it exists
			for (int i = menuPos + 1; i < menuOptions.size(); i++)
			{			
				MenuElement e = (MenuElement) menuOptions.elementAt(i);
				if (!e.isFaded())
				{
					menuPos = i;
					callingForm.repaint();
					return;
				}
			}
			
			// one more run through, for menu wrapping
			for (int i = 0; i < menuOptions.size(); i++)
			{			
				MenuElement e = (MenuElement) menuOptions.elementAt(i);
				if (!e.isFaded())
				{
					menuPos = i;
					callingForm.repaint();
					return;
				}
			}
		}
	}
	
	// check/uncheck option depending on current setting
	public void checkOption()
	{
		if (menuOptions == null) return; // not possible probably, but hey.
		MenuElement e = (MenuElement) menuOptions.elementAt(menuPos);
		
		if (e == null) return;
		
		if (e.isChecked())
			e.setChecked(false);
		else
			e.setChecked(true);
	}
	
	// is current option checked?
	public boolean currentOptionChecked()
	{
		if (menuOptions == null) return false; // not possible probably, but hey.
		
		MenuElement e = (MenuElement) menuOptions.elementAt(menuPos);
		if (e == null) return false;
		
		return e.isChecked();
	}
	
	// paint!
	public void paint(Graphics g, int bottomBorder)
	{
		if (menuOptions == null) return; // not possible probably, but hey.
		
		int totalOptionsHeight = 0;
		
		// determine the menu size
		for (int i = 0; i < menuOptions.size(); i++)
		{
			MenuElement e = (MenuElement) menuOptions.elementAt(i);
			if (e == null) continue;
			
			if (e.getLabel().equals(MENU_BREAK))
				totalOptionsHeight += 2;
			else
				totalOptionsHeight += MENU_OPTION_HEIGHT;
		}
		
		// draw menu bg
		g.setColor(255, 255, 255);
		g.fillRect((getWidth() / 2) - (MENU_OPTION_WIDTH / 2), getHeight() - bottomBorder - totalOptionsHeight - 2, MENU_OPTION_WIDTH, totalOptionsHeight);
		g.setColor(0,0,0);
		g.drawRect((getWidth() / 2) - (MENU_OPTION_WIDTH / 2) - 1, getHeight() - bottomBorder - totalOptionsHeight - 3, MENU_OPTION_WIDTH + 1, totalOptionsHeight + 1);
		
		// text color/font
		Font f = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_BOLD, Font.SIZE_MEDIUM);
		g.setFont(f);
		
		// increment for menu position based on if option is break or real option
		int optionHeightOffset = 0;

		// draw option text
		for (int i = 0; i < menuOptions.size(); i++)
		{
			MenuElement e =(MenuElement)menuOptions.elementAt(i);
			if (e == null) continue;
			
			drawMenuOption(g, e, totalOptionsHeight - optionHeightOffset, bottomBorder, e.isCheckable() && e.isChecked(), i);
			
			if (e.getLabel().equals(MENU_BREAK))
				optionHeightOffset += 2;
			else
				optionHeightOffset += MENU_OPTION_HEIGHT;
		}
	}
	
	public void drawMenuOption(Graphics g, MenuElement e, int topPos, int bottomBorder, boolean checked, int i)
	{
		// draw rect for selected if this is it
		if (menuPos == i)
		{
			g.setColor(192, 192, 255);
			g.fillRect((getWidth() / 2) - (MENU_OPTION_WIDTH / 2), getHeight() - bottomBorder - topPos - 2, MENU_OPTION_WIDTH, MENU_OPTION_HEIGHT);
		}
		
		int faded = 0;
		if (e.isFaded()) faded = Utilities.FONT_ALPHA50;
		
		Image img = null;
		if (checked) img = check;
		else if (((MenuElement) menuOptions.elementAt(i)).isCheckable()) img = nocheck;
		
		// draw check if option is checked and check image is loaded
		if (img != null)
			g.drawImage(img, (getWidth() / 2) - (MENU_OPTION_WIDTH / 2), getHeight() - bottomBorder - topPos - 1, Graphics.TOP | Graphics.LEFT);
		
		if (e.getLabel().equals(MENU_BREAK))
		{
			g.setColor(64, 64, 64);
			g.drawRect((getWidth() / 2) - (MENU_OPTION_WIDTH / 2), getHeight() - bottomBorder - topPos - 2, MENU_OPTION_WIDTH - 1, 1);
		}
		else
			Utilities.drawString(g, e.getLabel(), (getWidth() / 2) - (MENU_OPTION_WIDTH / 2) + 9, getHeight() - bottomBorder - topPos - 1, Graphics.TOP | Graphics.LEFT, mainProgram, Font.SIZE_SMALL | faded);
	}
	
	public int getWidth()
	{
		return (callingForm == null ? 0 : callingForm.getWidth());
	}
	
	public int getHeight()
	{
		return (callingForm == null ? 0 : callingForm.getHeight());
	}
	
	public void resetMenu()
	{
		if (menuOptions == null) return; // not possible probably, but hey.
		
		menuPos = -1;
		
		// find first non-faded option
		for (int i = 0; i < menuOptions.size(); i++)
		{
			MenuElement e = (MenuElement) menuOptions.elementAt(i);
			if (e == null) continue;
			if (!e.isFaded())
			{
				menuPos = i;
				return;
			}
		}
	}
}