// *****************************************
// (c) 2007 PBComic
// http://www.waicomic.com/pbcomic
//
// Nice little prog that lets people browse
// any comic that supports this system, via
// their mobile phone.
// Right now, it's only comics published
// with the PBComic system.
// *****************************************

package pbcviewer;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import javax.microedition.rms.RecordStore;

import java.io.*;
import java.util.*;

public class PBCViewer extends MIDlet implements CommandListener {
	// debug
	public static final boolean DEBUG_ON = false;
	private RecordStore debugRecords = null;
	
    private Display display;
    private static boolean isLoaded = false;
    
    public static final int SERVER_VERSION_SUPPORT = 1;
    public static final String CLIENT_VERSION = "0.9.1"; 
    
    private MMCanvas mm;
    
    // ugh, command for the normal forms
    private final Command goCmd = new Command("Submit", Command.SCREEN, 1);
    private final Command backCmd = new Command("Back", Command.EXIT, 2);
    private final Command ccBackCmd = new Command("", Command.BACK, 1);
    
    // blocks all actions until told otherwise
    private boolean blockActions = false;
    
	// date show option, if true it shows date in tiny box on top-center
	protected boolean showDate = true;
	
	public boolean dateShown() { return showDate; }
	public void setShowDate(boolean sd) { showDate = sd; }
	
	// hashtable to hold images
	public Hashtable smallFont;
	
	public PBCViewer() {
		// prevent from loading twice
		if (isLoaded)
		{
			if (mm.getComicCanvas() != null)
				display.setCurrent(mm.getComicCanvas());
			else
				display.setCurrent(mm);
			return;
		}
		
		// test
		try {
			String nully = null;
			nully.substring(0);
		} catch (Exception ex) {
			System.out.println("NPE caught!");
			System.out.println(ex);
			System.out.println(ex.getMessage());
			System.out.println(ex.toString());
		}
	
		//cOptions = new ChoiceGroup("Main Menu", Choice.EXCLUSIVE | Choice.TEXT_WRAP_OFF);
		display = Display.getDisplay(this);
		loadMenu();
		
		// load the fonts, die on failure
		try { loadFonts(); }
		catch (Exception ex)
		{  
			//System.out.println("Can't load fonts.");
			//try { destroyApp(false); } catch (Exception ey) { }
			//notifyDestroyed();
			//return;
		}
		
		// add listener
		isLoaded = true;
	}
	
	// get small letter
	public Image getSmallChar(char c)
	{
		return (Image) smallFont.get(Utilities.letterToPNG(c));
	}
	
	protected void loadFonts() throws Exception
	{
		// SMALL FONT FIRST
		// create hashtable
		smallFont = new Hashtable();
		
		for (char c = 'a'; c <= 'z'; c++)
			loadLetter(smallFont, c);
		for (char c = 'A'; c <= 'Z'; c++)
			loadLetter(smallFont, c);
		for (char c = '0'; c <= '9'; c++)
			loadLetter(smallFont, c);
		
		// other chars
		loadLetter(smallFont, '/');
		loadLetter(smallFont, '\\');
		loadLetter(smallFont, '?');
		loadLetter(smallFont, '!');
		loadLetter(smallFont, ':');
		loadLetter(smallFont, '.');
		loadLetter(smallFont, '\'');
		loadLetter(smallFont, '"');
		loadLetter(smallFont, '(');
		loadLetter(smallFont, ')');
		loadLetter(smallFont, ',');
		loadLetter(smallFont, '*');
		loadLetter(smallFont, '#');
		loadLetter(smallFont, Utilities.CHAR_ALPHA25);
		loadLetter(smallFont, Utilities.CHAR_ALPHA50);
		loadLetter(smallFont, Utilities.CHAR_ALPHA75);
		loadLetter(smallFont, ' ');
	}
	protected void loadLetter(Hashtable h, char c) throws Exception
	{
		//System.out.println("Loading " + c + " aka " + Utilities.letterToPNG(c));
		
		Image i = Image.createImage("/small/" + Utilities.letterToPNG(c));
		h.put(Utilities.letterToPNG(c), i);		
	}

	protected void destroyApp(boolean arg0) throws MIDletStateChangeException {
		try { mm.closeRecords(); }
		catch (Exception ex) { System.out.println(ex); }
		isLoaded = false;
	}
	
	public void killApp()
	{
		try { destroyApp(true); }
		catch (Exception ex) { }
		notifyDestroyed();
		return;
	}

	protected void pauseApp() {

	}

	protected void startApp() throws MIDletStateChangeException {
		focusMainMenu();		
	}
	
	protected void loadMenu()
	{
		mm = new MMCanvas(this);
	}
	
	// something needs this. simple get for "display"
	public Display getDisplay() { return display; }
	
	// some forms may need to block actions while something is processing
	// one example being HTTPForm.java
	public void setBlockActions(boolean ba) { blockActions = ba; }
	
	// set focus to main menu
	public void focusMainMenu()
	{
		display.setCurrent(mm);
	}
	
	public MMCanvas getMainMenu()
	{
		return mm;
	}
	
	public void addCommands(Form f)
	{
		f.addCommand(goCmd);
		f.addCommand(backCmd);
		f.setCommandListener(this);
	}

	public void commandAction(Command c, Displayable d) {
		Alert a = new Alert("Error!");
		a.setString("Error: You must enter a comic name!");
		a.setTimeout(Alert.FOREVER);
		
		if (c == backCmd)
		{
			display.setCurrent(mm);
		}
		else if (c == goCmd)
		{
			if (d == mm.getAddForm())
			{
				AddComicForm acf = (AddComicForm) d;
				
				// fail if comic has no name
				if (acf.getComicName().equals(""))
				{
					display.setCurrent(a, acf);
					return;
				}
				
				display.setCurrent(mm);
				
				mm.addNewComicRecord(acf.getComicName(), acf.getComicURL(), acf.getContentRating());
			}
			else if (d == mm.getModifyForm())
			{
				ModifyComicForm mcf = (ModifyComicForm) d;
				
				// fail if comic has no name
				if (mcf.getComicName().equals(""))
				{
					display.setCurrent(a, mcf);
					return;
				}
				
				display.setCurrent(mm);
				mm.modifyComicRecord(mcf.getComicName(), mcf.getComicURL(), mcf.getContentRating(), mcf.getRecordID());
			}
		}
		else if (c == ccBackCmd && d == mm.getComicCanvas())
		{ // close menu if menu is open
			if (mm.getComicCanvas().menuIsOpen())
				mm.getComicCanvas().closeMenu();
			else // otherwise, go back to main menu page
				mm.getComicCanvas().wrapUp(false);
		}
	}
	
	public GenericAlertDialog createAboutDialog(Canvas c)
	{
		return new GenericAlertDialog(c, this, "Pocket Comix " + CLIENT_VERSION + "\n"
				+ "(c) 2007 PBComic, all rights reserved.\n"
				+ "www.waicomic.com/pbcomic", (c.getWidth() / 8) * 7, true, false);
	}
	
	public void addBackCommand(ComicCanvas c)
	{
		c.addCommand(ccBackCmd);
		c.setCommandListener(this);
	}
	
	public void createDebug()
	{
		if (!DEBUG_ON) return;
		
		try {
			debugRecords = RecordStore.openRecordStore("PBCViewerDebug", true);
			debugRecords.setMode(RecordStore.AUTHMODE_PRIVATE, true);
		} catch (Exception ex) { }
	}
	
	public void addDebugRecord(String str)
	{
		createDebug();
		
		if (!DEBUG_ON) return;
		if (debugRecords == null) return;
		
		byte[] b = Utilities.charArrayToByteArray(str.toCharArray());
		try {
			debugRecords.addRecord(b, 0, b.length);
		} catch (Exception ex) { }
		
		try {
		if (debugRecords != null)
			debugRecords.closeRecordStore();
		} catch (Exception ex) {
			
		}
	}
	
	public GenericAlertDialog createDebugDialog(Canvas c)
	{
		createDebug();
		
		if (!DEBUG_ON) return null;
		if (debugRecords == null) return null;
		
		StringBuffer s = new StringBuffer("");

		try {
			int it = 0; // an iterator of sorts. bah.
			
			// get data from the record and pray it can be translated. lol
			for (int i = 1; it < debugRecords.getNumRecords(); i++)
			{
				byte[] b;
				
				// horrible i need to do this to get the current record. ugh
				try { b = debugRecords.getRecord(i); }
				catch (Exception ex) { continue; }
				
				s.append(Utilities.byteArrayToCharArray(b));
				s.append("\n");
				
				// if it gets here, record found. increase "it"
				it++;
			}
		} catch (Exception ey) {
			
		}
		
		return new GenericAlertDialog(c, this, s.toString(), (c.getWidth() / 8) * 7, true, false);
	} 
}
