// my simple point object which holds an x/y coordinate.
// programmer can specify value limitations which allow for lazier coding :)

package pbcviewer;

public class Point extends Object {
	private int x;
	private int y;
	private int upperLimit = -2000000000;
	private int leftLimit = -2000000000;
	private int lowerLimit = 2000000000;
	private int rightLimit = 2000000000;
	
	private Point() { }
	
	private void enforceLimits()
	{
		if (x < leftLimit)
			x = leftLimit;
		else if (x > rightLimit)
			x = rightLimit;
		
		if (y < upperLimit)
			y = upperLimit;
		else if (y > lowerLimit)
			y = lowerLimit;
	}
	
	public Point(int newX, int newY) { x = newX; y = newY; }
	public Point(int newX, int newY, int lLimit, int uLimit, int rLimit, int dLimit)
	{
		x = newX; y = newY;
		
		// fix invalid limits automatically
		if (uLimit > dLimit)
			uLimit = dLimit;
		if (lLimit > rLimit)
			lLimit = rLimit;
		
		upperLimit = uLimit; lowerLimit = dLimit;
		leftLimit = lLimit; rightLimit = rLimit;
		
		enforceLimits();
	}
	
	public void set(int newX, int newY) { x = newX; y = newY; }
	
	public int getX() { return x; }
	public int getY() { return y; }
	public void setX(int newX) { x = newX; enforceLimits(); }
	public void setY(int newY) { y = newY; enforceLimits(); }
	
	public void incX(int mod) { x += mod; enforceLimits(); }
	public void incY(int mod) { y += mod; enforceLimits(); }
	
	// these are great time savers for things like image canvas
	// returns whether or not vertical/horizontal limits are equal
	// if they are, I have no need to draw scrollbars.
	public boolean verticalLimitsEqual() { return upperLimit == lowerLimit; }
	public boolean horizontalLimitsEqual() { return leftLimit == rightLimit; }
}