// *****************************************
// (c) 2007 PBComic
// http://www.waicomic.com/pbcomic
//
// A bunch of static methods that are generally useful.
// *****************************************

package pbcviewer;

import java.util.Vector;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

public class Utilities {
	public static final int FONT_ALPHA25 = 32;
	public static final int FONT_ALPHA50 = 64;
	public static final int FONT_ALPHA75 = 128;
	public static final char CHAR_ALPHA25 = '%';
	public static final char CHAR_ALPHA50 = '^';
	public static final char CHAR_ALPHA75 = '}';
	
	public static byte[] charArrayToByteArray(char[] c)
	{
		byte[] b = new byte[c.length];
		for (int i = 0; i < c.length; i++)
			b[i] = (byte) c[i];
		
		return b;
	}
	
	public static char[] byteArrayToCharArray(byte[] c)
	{
		char[] b = new char[c.length];
		for (int i = 0; i < c.length; i++)
			b[i] = (char) c[i];
		
		return b;
	}
	
	// returns date string in U.S. human readable format
	public static final String humanFormatDate(String date)
	{
		if (date == null) return "";
		
		if (date.length() != 8) return "";
		
		StringBuffer newStr = new StringBuffer("");
		String year = date.substring(0, 4);
		String month = date.substring(4, 6);
		String day = date.substring(6, 8);
		
		newStr.append((month.startsWith("0") ? month.substring(1) : month));
		newStr.append("/");
		newStr.append((day.startsWith("0") ? day.substring(1) : day));
		newStr.append("/");
		newStr.append(year.substring(2));
		
		return newStr.toString();
	}
	
	public static final String letterToPNG(char c)
	{
		// bs char array crap. argh.
		char[] carr = new char[1];
		carr[0] = c;
		
		if (c >= 'a' && c <= 'z')
			return "l" + new String(carr) + ".png";
		else if (c >= 'A' && c <= 'Z')
			return "u" + (new String(carr) + ".png").toLowerCase();
		else if (c >= '0' && c <= '9')
			return new String(carr) + ".png";
		else if (c == '/')
			return "fslash.png";
		else if (c == '\\')
			return "bslash.png";
		else if (c == '.')
			return "period.png";
		else if (c == '?')
			return "question.png";
		else if (c == '!')
			return "exclaim.png";
		else if (c == ':')
			return "colon.png";
		else if (c == '\'')
			return "1quote.png";
		else if (c == '"')
			return "2quote.png";
		else if (c == '(')
			return "parento.png";
		else if (c == ')')
			return "parentc.png";
		else if (c == ',')
			return "comma.png";
		else if (c == '#')
			return "pound.png";
		else if (c == '*')
			return "star.png";
		else if (c == CHAR_ALPHA25)
			return "alpha25.png";
		else if (c == CHAR_ALPHA50)
			return "alpha50.png";
		else if (c == CHAR_ALPHA75)
			return "alpha75.png";
		
		// default
		return "space.png";
	}
	
	// my own drawstring, which supports the same stuff that the Graphics one does
	public static void drawString(Graphics g, String str, int w, int l, int flags, PBCViewer mainProgram, int font)
	{
		if (str == null || g == null || mainProgram == null) return;
		
		// use preloaded images saved in a hash in the main program
		if ((font & Font.SIZE_SMALL) != 0)
		{
			char[] strCA = str.toCharArray();
			
			int pos = 0; // applies if flags include Graphics.LEFT
			
			// determine start pos...does it have hcenter?
			if ((flags & Graphics.HCENTER) != 0)
				pos = (str.length() * 5) / -2;
			else if ((flags & Graphics.RIGHT) != 0) // is it right?
				pos = (str.length() * 5) * -1;
				
			
			for (int i = 0; i < str.length(); i++)
			{
				g.drawImage(mainProgram.getSmallChar(strCA[i]), w + pos, l, flags);
				// draw alphas (lighteners) if necessary
				if ((font & FONT_ALPHA25) != 0)
					g.drawImage(mainProgram.getSmallChar(CHAR_ALPHA25), w + pos, l, flags);
				else if ((font & FONT_ALPHA50) != 0)
					g.drawImage(mainProgram.getSmallChar(CHAR_ALPHA50), w + pos, l, flags);
				else if ((font & FONT_ALPHA75) != 0)
					g.drawImage(mainProgram.getSmallChar(CHAR_ALPHA75), w + pos, l, flags);
				
				pos += 5;
			}
		}
	}
	
	// breaks a string, based on the width allowed, into lines that'll fill a vector
	public static void strFillVector(String str, int w, Vector finalLines)
	{
		Vector lines = new Vector();
		
		if (str == null) str = "";
		
		if (finalLines == null) finalLines = new Vector();
		int pos = 0;
		int oldpos = 0;
		
		while (true) // break inside
		{
			if (pos == 0)
				oldpos = 0;
			else
				oldpos = pos + 1;
			pos = str.indexOf("\n", pos + 1);
			
			if (pos == -1)
			{
				lines.addElement(str.substring(oldpos, str.length()));
				break;
			}
			else
				lines.addElement(str.substring(oldpos, pos));
		}
		
		// num linebreaks known, find out how many more are needed to wrap to screen and
		// fill finalLines accordingly
		int maxLettersPerLine = w / 5;
		for (int i = 0; i < lines.size(); i++)
		{
			pos = 0;
			oldpos = 0;
			
			String s = (String) lines.elementAt(i);
			if (s.length() <= maxLettersPerLine) // str is fine. add it to final lines
				finalLines.addElement(s);
			else // needs to be divided into multiple lines, and inc the numlines counter
			{
				int startat = 0;
				
				while (pos < s.length())
				{	
					startat = pos + 1;
					
					StringBuffer thisLine = new StringBuffer("");
					
					while(true) // break inside
					{
						//if (pos == 0)
							oldpos = pos;
						//else
						//	oldpos = pos + 1;
						pos = s.indexOf(" ", pos + 1);
						
						// if pos is -1, set pos to str length
						if (pos == -1) pos = s.length();
						
						// 28+ char mess, stop at 28
						if (pos - startat > maxLettersPerLine && thisLine.toString().length() == 0)
						{
							thisLine.append(s.substring(oldpos, oldpos + maxLettersPerLine + 1));
							startat += maxLettersPerLine;
							pos = startat;
							break;
						} // too much for this line. don't add a line. 
						else if (pos - startat >= maxLettersPerLine)
						{
							pos = oldpos;
							startat = pos;
							break;
						}
						else // okay, append the stringbuffer and keep looping
						{
							//System.out.println("s.substring("+oldpos+", "+pos+") & s.length() = " + s.length());
							
							thisLine.append(s.substring(oldpos, pos));
							
							if (pos == s.length())
								break;
						}
					}
					
					if (thisLine.toString().startsWith(" "))
						finalLines.addElement(thisLine.toString().substring(1));
					else
						finalLines.addElement(thisLine.toString());
				}
			}
		}
	}
	
	public static int strpos(String haystack, String needle, int startpos)
	{
		// check for nulls
		if (haystack == null || needle == null)
			return -1;
		
		// blanks just as bad
		if (haystack == "" || needle == "")
			return -1;
		
		// easiest way, convert these to char arrays
		char[] h = haystack.toCharArray();
		char[] n = needle.toCharArray();
		
		for (int i = startpos; i < haystack.length() - needle.length() + 1; i++)
		{
			if (h[i] == n[0])
			{
				for (int j = 0; j < needle.length(); j++)
				{
					if (h[i + j] != n[j])
						break;
					
					if (j == needle.length() - 1) // string found!
						return i; // i should be correct position
				}
			}
		}
		
		// default
		return -1;
	}
	
	public static int strpos(String haystack, String needle)
	{
		return strpos(haystack, needle, 0);
	}
}